<?php
namespace Quivers\Integration\Model;

use Stripe\Stripe;

class QuiversStripe extends \Magento\Payment\Model\Method\Cc
{

    protected $_code = 'quivers_integration';
 
    protected $_canAuthorize = true;
    protected $_isGateway = true;

    protected $_stripeApi = false;
    protected $_scopeConfig;


    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate
        );
        $this->_scopeConfig = $scopeConfig;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_stripe/secret_key", $storeScope);
        $this->_minAmount = $this->_scopeConfig->getValue("payment/quivers_stripe/min_order_total", $storeScope);

        Stripe::setApiKey($secret_key);
    }

    public function assignData(\Magento\Framework\DataObject $data)
    {
        parent::assignData($data);

        $additionalData = $data->getAdditionalData();
        if (is_array($additionalData))
            $data->setData(array_merge($data->getData(), $additionalData));

        $info = $this->getInfoInstance();
        $token = $data['cc_stripejs_token'];
        $saveMyCard = $data['cc_save_card'];
        $info->setAdditionalInformation('cc_stripejs_token', $token);
        $info->setAdditionalInformation('cc_save_card', $saveMyCard);
        return $this;
    }
 
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $payment_token = $payment->getAdditionalInformation('cc_stripejs_token');
        $saveMyCard = $payment->getAdditionalInformation('cc_save_card');
        $order = $payment->getOrder();
        $currency = strtolower($order->getBaseCurrencyCode());
        $chargeAmount = $this->formatCurrencyAmount($amount, $currency);

        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();
        
        try {
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));

            if(count($customersList["data"])<=0) {
                $customer = \Stripe\Customer::create(array(
                'source' => $payment_token,
                'email' => $customerEmail,
                ));

                $customerId = $customer->id;
            }
            else {
                $customer = $customersList["data"][0];
                $customerId = $customer["id"];
                $customer = \Stripe\Customer::retrieve($customerId);
                if ($saveMyCard) {  // checked
                    try{
                        $customer->sources->create(array("source" => $payment_token));
                    }catch(\Exception $e){
                    }
                }
                $customerId = $customer->id;
            }
            
            $methods = \Magento\Framework\App\ObjectManager::getInstance();
            $customerSession = $methods->get('Magento\Customer\Model\Session');

            $customer = $customerSession->getCustomer();
            $myCustomerId = $customer->getId();
            if(isset($customerId) && $myCustomerId) {
                $customer->setQuiversStripeCustomerId($customerId);
                $customer->save();
            }
            $statement_descriptor = $payment->getAdditionalInformation('payment_descriptor');
            $charge = \Stripe\Charge::create(array(
                'customer' => $customerId,
                'source' => $payment_token,
                'capture' => false,
                'amount' => $chargeAmount,
                'currency' => strtolower($order->getBaseCurrencyCode()),
                'description' => sprintf('#%s, %s', $order->getIncrementId(), $order->getCustomerEmail()),
                'statement_descriptor' => $statement_descriptor,
            ));
        } catch (\Exception $e) {
            $this->_logger->error(__($e));
            throw new \Magento\Framework\Validator\Exception(__('Payment capturing error.'));
        }
        $payment
            ->setTransactionId($charge->id)
            ->setIsTransactionClosed(0);
        return $this;
    }

    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...

        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }



    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote && $quote->getBaseGrandTotal() <= $this->_minAmount) {
            return false;
        }
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_stripe/secret_key", $storeScope);

        if (!$secret_key) {
            return false;
        }
        return true;
    }
    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        // validation is already handled by stripejs
        return true;
    }

}
