<?php
namespace Quivers\Integration\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\CouldNotSaveException;
use function PHPSTORM_META\type;

class QuiversCustomTax implements ObserverInterface
{
    protected $taxEndpoint = '/CustomerOrders/Validate';
    protected $countriesEndpoint = '/countries';
    const XML_MARKETPLACE_ID = 'quivers/parameters/marketplaceId';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';

    private $logger;
    protected $curlClient;
    protected $scopeConfig;

    public function __construct(\Psr\Log\LoggerInterface $logger,
                                \Magento\Framework\HTTP\Client\Curl $curl,
                                \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig)
    {
        $this->curlClient = $curl;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
    }

    public function execute(Observer $observer)
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $marketPlace = $this->scopeConfig->getValue(self::XML_MARKETPLACE_ID, $storeScope);
        $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        if($quiversSandbox == 0) {
            $baseUrl = 'https://api.quivers.com/v1';
        } 
        else {
            $baseUrl = 'https://api.quiversdemo.com/v1';
        }
        $taxUrl = $baseUrl.$this->taxEndpoint;

        if(!$marketPlace) {
            return $this;
        }

        try {
            $quote = $observer->getQuote();
        } catch (LocalizedException $e) {
            return $this;
        }

        $items = array();
        $productMapping = [];

        # if no item selected then no need to proceed further.
        if(count($quote->getAllItems()) <= 0) {
            return $this;
        }

        $shipping_address = $quote->getShippingAddress();

        if(!$shipping_address) {
            return $this;
        }

        $country = $shipping_address->getCountryId();

        if(!$country) {
            return $this;
        }

        if(!$shipping_address->getRegionCode()) {
            return $this;
        }

        $shippingFeesTotal =  0.0;
        $shippingFeePerItem = 0.0;
        $shippingMethod = $shipping_address->getShippingMethod();
        if($shippingMethod) {
            $shippingRateData = $shipping_address->getShippingRatesCollection()->getData();
            if(count($shippingRateData)>0) {
                foreach ($shippingRateData as $rate) {
                    if($shippingMethod == $rate['code']) {
                        $shippingFeesTotal = $rate["price"];
                    }
                }
            }

            $totalItems = 0;
            foreach ($quote->getAllItems() as $quoteItem) {
                $itemData = $quoteItem->getData();
                if(array_key_exists('parent_item_id', $itemData) 
                    && !is_null($itemData['parent_item_id'])) {
                    continue;
                }
                $totalItems += 1;
            }

            $shippingFeePerItem = (float)$shippingFeesTotal/$totalItems;

        }

        foreach ($quote->getAllItems() as $quoteItem) {

            $itemData = $quoteItem->getData();
            // $this->logger->info("item data ", ["" => $itemData]);
            // check for parent item, if exists skip
            if(array_key_exists('parent_item_id', $itemData) 
                && !is_null($itemData['parent_item_id'])) {
                // $this->logger->info("parent_item exists ", ["" => $itemData]);
                continue;
            }
            $itemProduct = $quoteItem->getProduct();

            $itemId = (int)$itemProduct->getId();

            $productMapping[$itemId] = $quoteItem;

            $itemName = $itemProduct->getName();
            $itemVariantName = $itemProduct->getSku();
            $options = array();
            $options = $quoteItem->getProduct()->getTypeInstance(true)->getOrderOptions($quoteItem->getProduct());
            $options_arr = [];
            if (isset($options['options']) && !empty($options['options'])) {
                foreach ($options['options'] as $option) {
                    $optionTitle = $option['label'];
                    $optionId = $option['option_id'];
                    $optionValue = $option['value'];
                array_push($options_arr, array(
                    "name" => $optionTitle,
                    "value" => array("name" => $optionValue)
                ));

                }
            }

            $unitPrice = $itemProduct->getPrice();

            if(array_key_exists('base_price', $itemData)) {
                $unitPrice = $itemData['base_price'];
            }

            $totalDiscount = $quoteItem->getTotalDiscountAmount();

            $discounts = [];
            if($totalDiscount > 0 ) {
                array_push($discounts, [
                    "name"=> "Total Discount",
                    "amount"=> -$totalDiscount,
                ]);
            }

            $shippingFees = [];
            if($shippingFeePerItem > 0 ) {
                $shippingFees = [
                    "name" => $shippingMethod,
                    "amount" => $shippingFeePerItem
                ];
            }

            if(count($shippingFees) > 0 ) {
                $shippingFees = [$shippingFees];
            }

            array_push($items, array(
            "product" => array(
                "name" => $itemName,
                "refId" => $itemId,
                "variant" => array(
                    "name" => $itemVariantName,
                    "options" => $options_arr
                )
            ),

            "quantity" => $quoteItem->getQty(),
            "pricing" => array(
                    "unitPrice" => (float)$unitPrice,
                    "shippingFees" => $shippingFees,
                    "discounts"=> $discounts
                )
            ));

        }

        $taxData = [
            "marketplaceId" => $marketPlace,
            "shippingAddress" => [
                "phoneNumber" => $shipping_address->getTelephone(),
                "line1" => $shipping_address->getStreetLine(1),
                "line2" => $shipping_address->getStreetLine(2),
                "city" => $shipping_address->getCity(),
                "postCode" => $shipping_address->getPostcode(),
                "country" => $country,
                "region" => $shipping_address->getRegionCode()
            ],
            "items" => $items
        ];

        // $this->logger->info("taxData ", ["" => $ftaxData]);

        $this->curlClient->addHeader("Content-Type", "application/json");
        $this->curlClient->post($taxUrl, json_encode($taxData));
        $response = $this->curlClient->getBody();
        $totalTax = 0;
        $payment_descriptor = "";

        if($this->curlClient->getStatus()==200) {
            $responseDecoded = json_decode($response, true);
            $totalTax = $responseDecoded["result"]["totals"]["taxes"]["additional"];
            $subtotal = $responseDecoded["result"]["totals"]["subtotal"];
            $totalAmount = $responseDecoded["result"]["totals"]["total"];
            $payment_descriptor = $responseDecoded["result"]["statementDescriptor"];
            foreach ($responseDecoded["result"]["items"] as $responseItems) {
                $productId = $responseItems['productRefId'];

                $quoteItem = $productMapping[$productId];
                $quoteItemData = $quoteItem->getData();
                $appliedTaxes = $responseItems['pricing']['taxes'];
                if(count($responseItems['pricing']['taxes']) <= 0) {
                    continue;
                }
                $tax_amount = 0;
                $tax_percent = 0;
                foreach ($appliedTaxes as $appliedTax) {
                    $tax_amount += $appliedTax['amount'];
                    $tax_percent += $appliedTax['rate']*100;
                }
                $quoteItemData['tax_amount'] = $tax_amount;
                $quoteItemData['tax_percent'] = $tax_percent;
                $quoteItem->setData($quoteItemData);
            }
            $total = $observer->getData('total');
            $total->setTotalAmount('tax', $totalTax);
            $total->setBaseTotalAmount('tax', $totalTax);
            $total->setTotalAmount('subtotal', $subtotal);
            $total->setBaseTotalAmount('subtotal', $totalAmount);
            // update overall total
            $total->setGrandTotal($totalAmount);
            $total->setBaseGrandTotal($totalAmount);

            $total->setTotalAmount('total', $totalAmount);
            // $this->logger->info("responseDecoded", ["" => $total]);

            // get payment and set payment_descriptor as additional information
            $payment = $quote->getPayment();
            $payment->setAdditionalInformation('payment_descriptor', $payment_descriptor);

        }
        else {

            $this->tax_calculation_via_countries_api($baseUrl, $this->countriesEndpoint, $quote, $observer);
            
        }  
        return $this;
    }

    private function tax_calculation_via_countries_api($baseUrl, $countriesEndpoint, $quote, $observer) {
        $countriesUrl = $baseUrl.$countriesEndpoint;
        $this->logger->info($countriesUrl);

        $this->curlClient->addHeader("Content-Type", "application/json");
        $this->curlClient->get($countriesUrl);
        $result = $this->curlClient->getBody();
        $response = json_decode($result, true);

        $address = $quote->getShippingAddress();
        $shipping_amount = $quote->getShippingAddress()->getShippingAmount();

        foreach ($response['result'] as $country) {
            if ($country['abbreviations']['two']==$address->getCountryId()) {
                foreach ($country["regions"] as $region) {
                    if ($region['abbreviation']==$address->getRegionCode()){
                        $tax_rate = $region['maxTaxRate'];
                        $this->logger->info($tax_rate);
                        break;
                    }
                }
                break;
            }
            else {
                $tax_rate = 0;
            }
        }

        $totalTax = 0; // total tax
        $subtotal = 0; // order total excluding tax
        $totalAmount = 0; // order total including tax
        foreach ($quote->getAllItems() as $quoteItem) {
            $parent = $quoteItem->getParentItem();
            if($parent) {
                continue;
            }

            $itemDiscount = $quoteItem->getDiscountAmount();
            $itemQuantity = $quoteItem->getQty();
            $itemProduct = $quoteItem->getProduct();
            $itemUnitPrice = (float)$itemProduct->getPrice(); // item_price
            $quoteItemData = $quoteItem->getData();
            if(array_key_exists('base_price', $quoteItemData)) {
                $itemUnitPrice = $quoteItemData['base_price'];
            }
            $itemSubotal = $itemUnitPrice * $itemQuantity;
            $taxableAmount = $itemSubotal - $itemDiscount;
            $tax_amount = $tax_rate * $taxableAmount;
            $itemTotal = $taxableAmount + $tax_amount; // item_price_with_tax - shipping_fee_per_item
            
            $quoteItemData['price_incl_tax'] = $itemTotal/$itemQuantity;
            $quoteItemData['base_price_incl_tax'] = $itemTotal/$itemQuantity;
            $quoteItemData['tax_amount'] = $tax_amount;
            $quoteItemData['base_tax_amount'] = $tax_amount;
            $quoteItemData['tax_percent'] = $tax_rate * 100;
            $quoteItemData['row_total_incl_tax'] = $itemTotal;
            $quoteItem->setData($quoteItemData);

            $totalTax = $totalTax + $tax_amount;
            $subtotal = $subtotal + $itemSubotal;
            $totalAmount = $totalAmount + $itemTotal;

        }
        
        $totalAmount = $totalAmount + $shipping_amount;
        $total = $observer->getData('total');
        $total->setTotalAmount('tax', $totalTax);
        $total->setBaseTotalAmount('tax', $totalTax);
        $total->setTotalAmount('subtotal', $subtotal);
        $total->setBaseTotalAmount('subtotal', $totalAmount);
        // update overall total
        $total->setGrandTotal($totalAmount);
        $total->setBaseGrandTotal($totalAmount);
        $total->setTotalAmount('total', $totalAmount);

    }
}
